#include <iostream>
#include <vector>
using namespace std;

class CPolygon {
  protected:
    int width, height;
  public:
    void set_values (int a, int b) {
       width=a; height=b; 
    }
    // virtual pode ser redefinido em diversas classes derivadas
    // e como se na classe CPolygon esse metodo nao fosse util
    virtual int area () {
       return 0;
    }
    
    virtual CPolygon *clone() const {
       return new CPolygon(*this);
    }
    
    virtual int getValue() {
	return -1;
    }
};

class CRectangle: public CPolygon {
  private:
    int x;
    
  public:
    // metodo sobrecarregado (override))
    int area ()
      { return (width * height); }
      
    CRectangle(int x) {
	this->x = x; 
    }
    
    CRectangle (CRectangle const &source) {
	this->set_values(source.width, source.height);
	this->x = source.x;
    }
      
    CRectangle *clone() const {
	return new CRectangle(*this);
    }
    
    int getValue () {
	return x;
    }

};

class CTriangle: public CPolygon {
  private:
    double y;
    
  public:
    // metodo sobrecarregado (override))
    int area ()
      { return (width * height / 2); }
      
  
    CTriangle(double y) {
	this->y = y; 
    }
    
    CTriangle (CTriangle const &source) {
	this->set_values(source.width, source.height);
	this->y = source.y;
    }
      
    CTriangle *clone() const {
	return new CTriangle(*this);
    }
    
    int getValue () {
	return (int)y;
    }      
};

int main (int argc, char *argv[]) {

  CRectangle *a, *b;
  CPolygon *c;
  
  a = new CRectangle(199);
  // chama explicitamente o contrutor de copia
  b = new CRectangle(*a);
  // ocorre polimorfismo, e clone chama o 
  // construtor de copia apropriado
  c = a->clone();
  delete a;
  cout << c->getValue() << endl;
  cout << b->getValue() << endl << endl;
  
  CTriangle *d;
  d = new CTriangle(4.99);
  delete c;
  
  // faz uma copia de 'd' em 'c'
  c = d->clone();
  cout << c->getValue() << endl << endl;
  
  b->set_values(4,5);
  d->set_values(4,5);
  c->set_values(4,5);
  
  vector<CPolygon *> polys;
  polys.push_back(b->clone()); // ponteiro para retangulo
  polys.push_back(d->clone()); // ponteiro para triangulo
  
  // ponteiro para poligono que aponta para triangulo
  polys.push_back(c->clone()); 
  
  vector<CPolygon *>:: iterator it;
  for (it = polys.begin(); it != polys.end(); ++it) {
     cout << (*it)->area() << " value= " << (*it)->getValue() << endl; 
  }
  
  return 0;
}



